'use strict';

const electron = require('electron');
var Menu = electron.Menu;
// Manages reading and writing of .par files and other data
var file_manager = require('./js/file_manager');
// Module to control application life.
const app = electron.app;
// Module to create native browser window.
const BrowserWindow = electron.BrowserWindow;

// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the JavaScript object is garbage collected.
let mainWindow;


function createWindow () {
  // Create the browser window.
  mainWindow = new BrowserWindow({width: 1125, height: 8375});

  file_manager.gatherPathFiles(__dirname + '/config.json', function(result) {
       if (result.pathsAreMissing) {
           mainWindow.loadURL('file://' + __dirname + '/settings.html');
       } else {
           mainWindow.loadURL('file://' + __dirname + '/edit_config.html');
       }
   });

   // Template from Marco Pracucci
   var template = [{
       label: "Application",
       submenu: [
           { label: "About Application", selector: "orderFrontStandardAboutPanel:" },
           { type: "separator" },
           { label: "Quit", accelerator: "Command+Q", click: function() { app.quit(); }}
       ]}, {
           label: "Edit",
           submenu: [
               { label: "Undo", accelerator: "CmdOrCtrl+Z", selector: "undo:" },
               { label: "Redo", accelerator: "Shift+CmdOrCtrl+Z", selector: "redo:" },
               { type: "separator" },
               { label: "Cut", accelerator: "CmdOrCtrl+X", selector: "cut:" },
               { label: "Copy", accelerator: "CmdOrCtrl+C", selector: "copy:" },
               { label: "Paste", accelerator: "CmdOrCtrl+V", selector: "paste:" },
               { label: "Select All", accelerator: "CmdOrCtrl+A", selector: "selectAll:" }
           ]}
       ];

  Menu.setApplicationMenu(Menu.buildFromTemplate(template));


  // Emitted when the window is closed.
  mainWindow.on('closed', function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });
}

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
app.on('ready', createWindow);

// Quit when all windows are closed.
app.on('window-all-closed', function () {
  // On OS X it is common for applications and their menu bar
  // to stay active until the user quits explicitly with Cmd + Q
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

app.on('activate', function () {
  // On OS X it's common to re-create a window in the app when the
  // dock icon is clicked and there are no other windows open.
  if (mainWindow === null) {
    createWindow();
  }
});
